@@ -22,7 +22,8 @@
     [clojure.lang PersistentQueue]
     [java.io File InputStreamReader BufferedReader]
     [java.lang ProcessBuilder ProcessBuilder$Redirect]
-    [java.util List]))
+    [java.util List]
+    [java.util.concurrent ExecutorService]))
 
 ;(set! *warn-on-reflection* true)
 
@@ -386,7 +387,7 @@
                 result @pend-children]
             (when (instance? Throwable result)
               (on-error result))
-            (next-path (->> result (filter (fn [[lib _coord]] (child-pred lib))) (map #(conj ppath %))) q' on-error))
+            (recur (->> result (filter (fn [[lib _coord]] (child-pred lib))) (map #(conj ppath %))) q' on-error))
           {:path next-q, :q' q'})))))
 
 (defn- expand-deps
@@ -498,6 +499,7 @@
                        cut-orphans)
          lib-map (lib-paths version-map)
          lib-map' (download-libs executor lib-map deps-map)]
+     (.shutdownNow ^ExecutorService executor)
      (with-meta lib-map' (meta version-map))))
   ;; deprecated arity, retained for backwards compatibility
   ([deps-map args-map settings]
@@ -784,6 +786,17 @@
       project-edn (assoc :project project-edn)
       extra-edn (assoc :extra extra-edn))))
 
+(defmacro ^:private in-project-dir
+  "If project deps.edn is not in the current dir, push project directory
+  into current directory context while creating basis. Local deps use paths
+  relative to project dir."
+  [project-deps & body]
+  `(if (and (instance? String ~project-deps)
+        (not (.equals dir/*the-dir* (.getParentFile (jio/file ~project-deps)))))
+     (dir/with-dir (.getParentFile (jio/file ~project-deps))
+        ~@body)
+     (do ~@body)))
+
 (defn create-basis
   "Create a basis from a set of deps sources and a set of aliases. By default, use
    root, user, and project deps and no argmaps (essentially the same classpath you get by
@@ -819,39 +832,31 @@
     :classpath - classpath map per make-classpath-map
     :classpath-roots - vector of paths in classpath order"
   [{:keys [root user project extra aliases] :as params}]
-  (let [basis-config (cond-> nil
-                       (contains? params :root) (assoc :root root)
-                       (contains? params :project) (assoc :project project)
-                       (contains? params :user) (assoc :user user)
-                       (contains? params :extra) (assoc :extra extra)
-                       (seq aliases) (assoc :aliases (vec aliases)))
+  (in-project-dir project
+    (let [basis-config (cond-> nil
+                         (contains? params :root) (assoc :root root)
+                         (contains? params :project) (assoc :project project)
+                         (contains? params :user) (assoc :user user)
+                         (contains? params :extra) (assoc :extra extra)
+                         (seq aliases) (assoc :aliases (vec aliases)))
 
-        {root-edn :root user-edn :user project-edn :project extra-edn :extra} (create-edn-maps params)
-        edn-maps [root-edn user-edn project-edn extra-edn]
+          {root-edn :root user-edn :user project-edn :project extra-edn :extra} (create-edn-maps params)
+          edn-maps [root-edn user-edn project-edn extra-edn]
 
-        alias-data (->> edn-maps
-                        (map :aliases)
+          alias-data (->> edn-maps
+                       (map :aliases)
+                       (remove nil?)
+                       (apply merge-with merge))
+          argmap-data (->> aliases
                         (remove nil?)
-                        (apply merge-with merge))
-        argmap-data (->> aliases
-                         (remove nil?)
-                         (map #(get alias-data %)))
-        argmap (apply merge-alias-maps argmap-data)
+                        (map #(get alias-data %)))
+          argmap (apply merge-alias-maps argmap-data)
 
-        project-tooled-edn (tool project-edn argmap)
-        merged-edn (merge-edns [root-edn user-edn project-tooled-edn extra-edn])
-        basis (calc-basis merged-edn {:resolve-args argmap, :classpath-args argmap})]
-    (cond-> (assoc basis :basis-config basis-config)
-      (pos? (count argmap)) (assoc :argmap argmap)
-
-      ;; *** NOTE:
-      ;; Historically the basis included :resolve-args and :classpath-args
-      ;; which happened to contain all of the argmap. For a while, this
-      ;; will continue to be the case while users migrate to using
-      ;; :argmap for all of those needs. Eventually remove these:
-      (pos? (count argmap)) (assoc :resolve-args argmap)
-      (pos? (count argmap)) (assoc :classpath-args argmap)
-     )))
+          project-tooled-edn (tool project-edn argmap)
+          merged-edn (merge-edns [root-edn user-edn project-tooled-edn extra-edn])
+          basis (calc-basis merged-edn {:resolve-args argmap, :classpath-args argmap})]
+      (cond-> (assoc basis :basis-config basis-config)
+        (pos? (count argmap)) (assoc :argmap argmap)))))
 
 (defn resolve-added-libs
   "Given an existing map of current libs and a map of libs to add,
@@ -884,6 +889,15 @@
                  (pos? (count conflict)) (assoc :conflict conflict))]
     result))
 
+(defn find-latest-version
+  "Find versions of lib across all registered procurer types, and return
+  latest version of coord from the first procurer type that has any coords,
+  starting with :mvn, then :git. If none are found, return nil."
+  [{:keys [lib procurer]}]
+  (let [types (distinct (into [:mvn :git] (ext/procurer-types)))]
+    (println "found versions: " (some #(ext/find-versions lib nil % procurer) types))
+    (last (some #(ext/find-versions lib nil % procurer) types))))
+
 ;; Load extensions
 (load "/clojure/tools/deps/extensions/maven")
 (load "/clojure/tools/deps/extensions/local")
@@ -894,6 +908,10 @@
 (comment
   (require '[clojure.tools.deps.util.maven :as mvn])
 
+  (find-latest-version {:lib 'org.clojure/data.json :procurer {:mvn/repos mvn/standard-repos}})
+  (find-latest-version {:lib 'io.github.clojure/tools.tools
+                        :procurer {:mvn/repos mvn/standard-repos}})
+
   (def ex-svc (concurrent/new-executor 2))
 
   (calc-basis
